require "minitest/autorun"
require "mcmailer"
require "uuid"
require "yaml"

describe MailchimpClient do
  Mcmailer.configure_with(File.expand_path('../../config/mcmailer.yml', __FILE__))
  MAILCHIMP = Mcmailer.config['test']
  
  def client
    @client ||= MailchimpClient.new(MAILCHIMP['api_key'], MAILCHIMP['timeout'])
  end
  
  def uuid
    @uuid ||= UUID.new
  end
  
  def random
    @random ||= Random.new
  end
  
  before do
    client.use_list(MAILCHIMP['list_name'])
    client.destroy_all_subscribers!
    
    client.subscribers.count.must_equal 0
  end
  
  it "can subscribe people to to the mailing list individually" do
    subscriber_email = 'mailchimp_client_test@unit.test'
    client.subscribe(subscriber_email, random_field_data)
    
    client.subscribers.count.must_equal 1
    client.subscribers.first['email'].must_equal subscriber_email
  end

  it "can destroy an individual subscription from the mailing list" do 
    subscriber_email = 'mailchimp_client_test@unit.test'
    client.subscribe(subscriber_email, random_field_data)
    
    client.subscribers.count.must_equal 1
    client.subscribers.first['email'].must_equal subscriber_email

    client.destroy!(subscriber_email)

    client.subscribers.count.must_equal 0
  end

  it "can subscribe a batch of users" do
    batch_size = 15
    subscribers = random_batch(batch_size)
    client.batch_subscribe(subscribers)

    client.subscribers.count.must_equal batch_size
  end

  it "has a batch subscription operation that's idempotent" do
    batch_size = 15
    subscribers = random_batch(batch_size)
    5.times do
      client.batch_subscribe(subscribers)
    end

    client.subscribers.count.must_equal batch_size
  end

  it "can destroy a batch of subscriptions" do
    batch_size = 15
    subscribers = random_batch(batch_size)
    client.batch_subscribe(subscribers)

    client.subscribers.count.must_equal batch_size
    
    unsubscription_batch_size = 13
    emails = emails_from_batch(subscribers)
    client.batch_destroy!(emails[0...unsubscription_batch_size])
    
    client.subscribers.count.must_equal (batch_size - unsubscription_batch_size)
  end

  def random_batch(count)
    (1..count).collect do |n|
      result = random_field_data
    end
  end

  def emails_from_batch(batch)
    batch.collect {|s| s[:EMAIL]}
  end

  def random_field_data()
    data = { :EMAIL => "#{uuid.generate}@unit.test" }
    fields = MAILCHIMP['required_fields']
    fields = fields.merge(MAILCHIMP['optional_fields']) if random.rand >= 0.5
    fields.each_pair do |name,type|
      data[name.upcase.to_sym] = random_value(type)
    end
    data
  end
  
  def random_value(type)
    case type
    when 'text'
      'Lorem ipsum'
    when 'number'
      random.rand(10)
    when 'bool'
      (random.rand >= 0.5) ? 1 : 0
    else
      ''
    end
  end
end
